searchData={"items":[{"type":"module","title":"Twirp","doc":"Twirp provides an elixir implementation of the [twirp rpc framework](https://github.com/twitchtv/twirp)\ndeveloped by Twitch. The protocol defines semantics for routing and\nserialization of RPCs based on protobufs.","ref":"Twirp.html"},{"type":"module","title":"Example - Twirp","doc":"The canonical Twirp example is a Haberdasher service. Here's the protobuf\ndescription for the service.\n\n```protobuf\nsyntax = \"proto3\";\n\npackage example;\n\n// Haberdasher service makes hats for clients.\nservice Haberdasher {\n  // MakeHat produces a hat of mysterious, randomly-selected color!\n  rpc MakeHat(Size) returns (Hat);\n}\n\n// Size of a Hat, in inches.\nmessage Size {\n  int32 inches = 1; // must be > 0\n}\n\n// A Hat is a piece of headwear made by a Haberdasher.\nmessage Hat {\n  int32 inches = 1;\n  string color = 2; // anything but \"invisible\"\n  string name = 3; // i.e. \"bowler\"\n}\n```\n\nWe'll assume for now that this proto file lives in `priv/protos/service.proto`","ref":"Twirp.html#module-example"},{"type":"module","title":"Code generation - Twirp","doc":"We can now use `protoc` to generate the files we need. You can run this command\nfrom the root directory of your project.\n\n    $ protoc --proto_path=./priv/protos --elixir_out=./lib/example --twirp_elixir_out=./lib/example ./priv/protos/service.proto\n\nAfter running this command there should be 2 files located in `lib/example`.\n\nThe message definitions:\n\n```elixir\ndefmodule Example.Size do\n  @moduledoc false\n  use Protobuf, syntax: :proto3\n\n  @type t :: %__MODULE__{\n          inches: integer\n        }\n\n  defstruct [:inches]\n\n  field :inches, 1, type: :int32\nend\n\ndefmodule Example.Hat do\n  @moduledoc false\n  use Protobuf, syntax: :proto3\n\n  @type t :: %__MODULE__{\n          inches: integer,\n          color: String.t(),\n          name: String.t()\n        }\n  defstruct [:inches, :color, :name]\n\n  field :inches, 1, type: :int32\n  field :color, 2, type: :string\n  field :name, 3, type: :string\nend\n```\n\nThe service and client definition:\n\n```elixir\ndefmodule Example.HaberdasherService do\n  @moduledoc false\n  use Twirp.Service\n\n  package \"example\"\n  service \"Haberdasher\"\n\n  rpc :MakeHat, Example.Size, Example.Hat, :make_hat\nend\n\ndefmodule Example.HaberdasherClient do\n  @moduledoc false\n  use Twirp.Client, service: Example.HaberdasherService\nend\n```","ref":"Twirp.html#module-code-generation"},{"type":"module","title":"Implementing the server - Twirp","doc":"Now that we've generated the service definition we can implement a \"handler\"\nmodule that will implement each \"method\".\n\n```elixir\ndefmodule Example.HaberdasherHandler do\n  @colors ~w|white black brown red blue|\n  @names [\"bowler\", \"baseball cap\", \"top hat\", \"derby\"]\n\n  def make_hat(_ctx, size) do\n    if size <= 0 do\n      Twirp.Error.invalid_argument(\"I can't make a hat that small!\")\n    else\n      %Haberdasher.Hat{\n        inches: size.inches,\n        color: Enum.random(@colors),\n        name: Enum.random(@names)\n      }\n    end\n  end\nend\n```\n\nSeparating the service and handler like this may seem a little odd but there are\ngood reasons to do this. The most important is that it allows the service to be\nautogenerated again in the future. The second reason is that it allows us to\neasily mock service implementations for testing.","ref":"Twirp.html#module-implementing-the-server"},{"type":"module","title":"Running the server - Twirp","doc":"To serve traffic Twirp provides a Plug. We use this plug to attach our service\ndefinition with our handler.\n\n```elixir\ndefmodule Example.Router do\n  use Plug.Router\n\n  plug Twirp.Plug,\n    service: Haberdasher.HatMakerService,\n    handler: Haberdasher.HatMakerHandler\nend\n```\n\n```elixir\ndefmodule Example.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      Plug.Cowboy.child_spec(scheme: :http, plug: Example.Router, options: [port: 4040]),\n    ]\n\n    opts = [strategy: :one_for_one, name: Example.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nIf you start your application your plug will now be available on port 4040.","ref":"Twirp.html#module-running-the-server"},{"type":"module","title":"Using the client - Twirp","doc":"Client definitions are generated alongside the service definition. This allows\nyou to generate clients for your services in other applications.  You can make\nRPC calls like so:\n\n```elixir\ndefmodule AnotherService.GetHats do\n  alias Example.HaberdasherClient, as: Client\n  alias Example.{Size, Hat}\n\n  def make_a_hat(inches) do\n    case Client.make_hat(Size.new(inches: inches)) do\n      {:ok, %Hat{}=hat} ->\n        hat\n\n      {:error, %Twirp.Error{msg: msg}} ->\n        Logger.error(msg)\n    end\n  end\nend\n```","ref":"Twirp.html#module-using-the-client"},{"type":"module","title":"Twirp.Plug","doc":"Provides a plug that takes service and handler module. If the request is\ndirected at the \"twirp\" endpoint then the plug will intercept the conn and\nprocess it. Otherwise it allows the conn to pass through. This is a deviation\nfrom the twirp specification but it allows users to include twirp services\ninto their existing plug stacks.\n\nYou can use the plug like so:\n\n```elixir\nplug Twirp.Plug,\n  service: MyService,\n  handler: MyHandler,\n```","ref":"Twirp.Plug.html"},{"type":"function","title":"Twirp.Plug.call/2","doc":"","ref":"Twirp.Plug.html#call/2"},{"type":"function","title":"Twirp.Plug.call_before_hooks/3","doc":"","ref":"Twirp.Plug.html#call_before_hooks/3"},{"type":"function","title":"Twirp.Plug.call_on_error_hooks/3","doc":"","ref":"Twirp.Plug.html#call_on_error_hooks/3"},{"type":"function","title":"Twirp.Plug.call_on_exception_hooks/3","doc":"","ref":"Twirp.Plug.html#call_on_exception_hooks/3"},{"type":"function","title":"Twirp.Plug.call_on_success_hooks/2","doc":"","ref":"Twirp.Plug.html#call_on_success_hooks/2"},{"type":"function","title":"Twirp.Plug.env_s/0","doc":"","ref":"Twirp.Plug.html#env_s/0"},{"type":"function","title":"Twirp.Plug.hook_result_s/0","doc":"","ref":"Twirp.Plug.html#hook_result_s/0"},{"type":"function","title":"Twirp.Plug.init/1","doc":"","ref":"Twirp.Plug.html#init/1"},{"type":"function","title":"Twirp.Plug.validate_req/3","doc":"","ref":"Twirp.Plug.html#validate_req/3"},{"type":"module","title":"Twirp.Service","doc":"Provides a DSL and some convenience functions for defining twirp service\ndefinitions.","ref":"Twirp.Service.html"},{"type":"function","title":"Twirp.Service.full_name/1","doc":"","ref":"Twirp.Service.html#full_name/1"},{"type":"macro","title":"Twirp.Service.package/1","doc":"","ref":"Twirp.Service.html#package/1"},{"type":"macro","title":"Twirp.Service.rpc/4","doc":"","ref":"Twirp.Service.html#rpc/4"},{"type":"function","title":"Twirp.Service.rpc_s/0","doc":"","ref":"Twirp.Service.html#rpc_s/0"},{"type":"function","title":"Twirp.Service.s/0","doc":"","ref":"Twirp.Service.html#s/0"},{"type":"macro","title":"Twirp.Service.service/1","doc":"","ref":"Twirp.Service.html#service/1"},{"type":"module","title":"Twirp.Telemetry","doc":"Provides telemetry for twirp clients and servers\n\nTwirp executes the following events:\n\n  * `[:twirp_elixir, :rpc, :start]` - Executed before making an rpc call to another service.\n\n  #### Measurements\n\n    * `:system_time` - The system time\n\n  #### Metadata\n\n    * `:client` - The client module issuing the call.\n    * `:method` - The RPC method\n    * `:service` - The url for the service\n\n  * `[:twirp_elixir, :rpc, :stop]` - Executed after a connection is retrieved from the pool.\n\n  #### Measurements\n\n    * `:duration` - Duration to send an rpc to a service and wait for a response.\n\n  #### Metadata\n\n    * `:client` - The client module issuing the call.\n    * `:method` - The RPC method\n    * `:service` - The url for the service\n    * `:error` - Optional key. If the call resulted in an error this key will be present along with the Twirp Error.\n\n  * `[:twirp_elixir, :call, :start]` - Executed before the twirp handler is called\n\n  #### Measurements\n\n    * `:system_time` - The system time\n\n  #### Metadata\n\n    There is no metadata for this event.\n\n  * `[:twirp_elixir, :call, :stop]` - Executed after twirp handler has been executed.\n\n  #### Measurements\n\n    * `:duration` - Duration to handle the rpc call.\n\n  #### Metadata\n\n    * `:content_type` - The content type being used, either proto or json.\n    * `:method` - The name of the method being executed.\n    * `:error` - Optional key. If the call resulted in an error this key will be present along with the Twirp Error.\n\n  * `[:twirp_elixir, :call, :exception]` - Executed if the twirp handler raises an exception\n\n  #### Measurements\n\n    * `:duration` - Duration to handle the rpc call.\n\n  #### Metadata\n\n    * `:kind` - The kind of error that was raised.\n    * `:error` - The exception\n    * `:stacktrace` - The stacktrace","ref":"Twirp.Telemetry.html"},{"type":"exception","title":"Twirp.Client.AdapterError","doc":"","ref":"Twirp.Client.AdapterError.html"},{"type":"exception","title":"Twirp.Error","doc":"This module defines the different error codes as specified in\nhttps://twitchtv.github.io/twirp/docs/spec_v5.html#error-codes.\n\nWe provide a function for each error code to that its easy for users to return\nerrors in their handlers.","ref":"Twirp.Error.html"},{"type":"function","title":"Twirp.Error.aborted/2","doc":"","ref":"Twirp.Error.html#aborted/2"},{"type":"function","title":"Twirp.Error.already_exists/2","doc":"","ref":"Twirp.Error.html#already_exists/2"},{"type":"function","title":"Twirp.Error.bad_route/2","doc":"","ref":"Twirp.Error.html#bad_route/2"},{"type":"function","title":"Twirp.Error.canceled/2","doc":"","ref":"Twirp.Error.html#canceled/2"},{"type":"function","title":"Twirp.Error.code_to_status/1","doc":"","ref":"Twirp.Error.html#code_to_status/1"},{"type":"function","title":"Twirp.Error.data_loss/2","doc":"","ref":"Twirp.Error.html#data_loss/2"},{"type":"function","title":"Twirp.Error.deadline_exceeded/2","doc":"","ref":"Twirp.Error.html#deadline_exceeded/2"},{"type":"function","title":"Twirp.Error.failed_precondition/2","doc":"","ref":"Twirp.Error.html#failed_precondition/2"},{"type":"function","title":"Twirp.Error.internal/2","doc":"","ref":"Twirp.Error.html#internal/2"},{"type":"function","title":"Twirp.Error.invalid_argument/2","doc":"","ref":"Twirp.Error.html#invalid_argument/2"},{"type":"function","title":"Twirp.Error.new/3","doc":"","ref":"Twirp.Error.html#new/3"},{"type":"function","title":"Twirp.Error.not_found/2","doc":"","ref":"Twirp.Error.html#not_found/2"},{"type":"function","title":"Twirp.Error.out_of_range/2","doc":"","ref":"Twirp.Error.html#out_of_range/2"},{"type":"function","title":"Twirp.Error.permission_denied/2","doc":"","ref":"Twirp.Error.html#permission_denied/2"},{"type":"function","title":"Twirp.Error.resource_exhausted/2","doc":"","ref":"Twirp.Error.html#resource_exhausted/2"},{"type":"function","title":"Twirp.Error.unauthenticated/2","doc":"","ref":"Twirp.Error.html#unauthenticated/2"},{"type":"function","title":"Twirp.Error.unavailable/2","doc":"","ref":"Twirp.Error.html#unavailable/2"},{"type":"function","title":"Twirp.Error.unimplemented/2","doc":"","ref":"Twirp.Error.html#unimplemented/2"},{"type":"function","title":"Twirp.Error.unknown/2","doc":"","ref":"Twirp.Error.html#unknown/2"},{"type":"function","title":"Twirp.Error.valid_code?/1","doc":"","ref":"Twirp.Error.html#valid_code?/1"},{"type":"type","title":"Twirp.Error.t/0","doc":"","ref":"Twirp.Error.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}